@@ -89,8 +89,7 @@ pagination_initialize_finished (GObject      *object,
                                 gpointer     user_data)
 {
     WebKitJavascriptResult *js_result;
-    JSValueRef              value;
-    JSGlobalContextRef      context;
+    JSCValue               *value;
     GError                 *error = NULL;
     GepubWidget            *widget = GEPUB_WIDGET (user_data);
 
@@ -101,12 +100,11 @@ pagination_initialize_finished (GObject      *object,
         return;
     }
 
-    context = webkit_javascript_result_get_global_context (js_result);
-    value = webkit_javascript_result_get_value (js_result);
-    if (JSValueIsNumber (context, value)) {
+    value = webkit_javascript_result_get_js_value (js_result);
+    if (jsc_value_is_number (value)) {
         double n;
 
-        n = JSValueToNumber (context, value, NULL);
+        n = jsc_value_to_double (value);
         widget->chapter_length = (int)n;
 
         if (widget->init_chapter_pos) {
@@ -132,8 +130,7 @@ get_length_finished (GObject      *object,
                      gpointer     user_data)
 {
     WebKitJavascriptResult *js_result;
-    JSValueRef              value;
-    JSGlobalContextRef      context;
+    JSCValue               *value;
     GError                 *error = NULL;
     GepubWidget            *widget = GEPUB_WIDGET (user_data);
 
@@ -144,12 +141,11 @@ get_length_finished (GObject      *object,
         return;
     }
 
-    context = webkit_javascript_result_get_global_context (js_result);
-    value = webkit_javascript_result_get_value (js_result);
-    if (JSValueIsNumber (context, value)) {
+    value = webkit_javascript_result_get_js_value (js_result);
+    if (jsc_value_is_number (value)) {
         double n;
 
-        n = JSValueToNumber (context, value, NULL);
+        n = jsc_value_to_double (value);
         widget->length = (int)n;
     } else {
         g_warning ("Error running javascript: unexpected return value");
@@ -230,6 +226,32 @@ reload_length_cb (GtkWidget *widget,
     }
 }
 
+static void
+set_current_chapter_by_uri (WebKitWebView  *web_view)
+{
+    GepubWidget *widget = GEPUB_WIDGET (web_view);
+    const gchar *uri_string;
+    GUri *uri;
+    const gchar *path;
+    gint chapter;
+
+    uri_string = webkit_web_view_get_uri (web_view);
+
+    if (g_strcmp0 ("about:blank", uri_string)) {
+        uri = g_uri_parse (uri_string, SOUP_HTTP_URI_FLAGS, NULL);
+        path = g_uri_get_path (uri);
+        chapter = gepub_doc_resource_uri_to_chapter (widget->doc, path);
+        gepub_doc_set_chapter (widget->doc, chapter);
+        g_uri_unref (uri);
+    }
+    // Else we're on the cover or table of contents (and can't tell which)
+    // but we can only get there through setting the chapter number
+    // so we don't need to do anything here
+}
+
+static void
+reload_current_chapter (GepubWidget *widget);
+
 static void
 docready_cb (WebKitWebView  *web_view,
              WebKitLoadEvent load_event,
@@ -239,6 +261,11 @@ docready_cb (WebKitWebView  *web_view,
 
     if (load_event == WEBKIT_LOAD_FINISHED) {
         reload_length_cb (GTK_WIDGET (widget), NULL, NULL);
+        g_signal_handlers_disconnect_by_func (widget->doc,
+                                              reload_current_chapter, widget);
+        set_current_chapter_by_uri (web_view);
+        g_signal_connect_swapped (widget->doc, "notify::chapter",
+                                  G_CALLBACK (reload_current_chapter), widget);
     }
 }
 
@@ -247,7 +274,6 @@ resource_callback (WebKitURISchemeRequest *request, gpointer user_data)
 {
     GInputStream *stream;
     gchar *path;
-    gchar *uri;
     gchar *mime;
     GepubWidget *widget = user_data;
     GBytes *contents;
@@ -255,9 +281,7 @@ resource_callback (WebKitURISchemeRequest *request, gpointer user_data)
     if (!widget->doc)
       return;
 
-    uri = g_strdup (webkit_uri_scheme_request_get_uri (request));
-    // removing "epub:///"
-    path = uri + 8;
+    path = g_strdup (webkit_uri_scheme_request_get_path (request));
     contents = gepub_doc_get_resource (widget->doc, path);
     mime = gepub_doc_get_resource_mime (widget->doc, path);
 
@@ -279,7 +303,7 @@ resource_callback (WebKitURISchemeRequest *request, gpointer user_data)
     g_object_unref (stream);
     g_bytes_unref (contents);
     g_free (mime);
-    g_free (uri);
+    g_free (path);
 }
 
 static void
@@ -364,16 +388,45 @@ gepub_widget_init (GepubWidget *widget)
     widget->line_height = 0;
 }
 
+static GObject *
+gepub_widget_constructor (GType                  gtype,
+                          guint                  n_properties,
+                          GObjectConstructParam *properties)
+{
+  GObject *object;
+  GObjectClass *parent_class;
+
+  parent_class = G_OBJECT_CLASS (gepub_widget_parent_class);
+  object = parent_class->constructor (gtype, n_properties, properties);
+
+  g_object_set (object,
+                "web-context", g_object_new (WEBKIT_TYPE_WEB_CONTEXT, NULL),
+                NULL);
+
+  return object;
+}
+
+
 static void
 gepub_widget_constructed (GObject *object)
 {
     WebKitWebContext *ctx;
+    WebKitSettings *settings;
     GepubWidget *widget = GEPUB_WIDGET (object);
 
     G_OBJECT_CLASS (gepub_widget_parent_class)->constructed (object);
 
     ctx = webkit_web_view_get_context (WEBKIT_WEB_VIEW (widget));
     webkit_web_context_register_uri_scheme (ctx, "epub", resource_callback, widget, NULL);
+
+    settings = webkit_web_view_get_settings (WEBKIT_WEB_VIEW (widget));
+    g_object_set (G_OBJECT (settings),
+                  "enable-javascript-markup", FALSE,
+                  "enable-java", FALSE,
+                  "javascript-can-access-clipboard", FALSE,
+                  "javascript-can-open-windows-automatically", FALSE,
+                  NULL);
+
     g_signal_connect (widget, "load-changed", G_CALLBACK (docready_cb), NULL);
     g_signal_connect (widget, "size-allocate", G_CALLBACK (reload_length_cb), NULL);
 }
@@ -383,6 +436,7 @@ gepub_widget_class_init (GepubWidgetClass *klass)
 {
     GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+    object_class->constructor = gepub_widget_constructor;
     object_class->constructed = gepub_widget_constructed;
     object_class->finalize = gepub_widget_finalize;
     object_class->set_property = gepub_widget_set_property;
